@@ -36,9 +36,10 @@ def tuxauth(mocker):
     get = mocker.Mock(
         return_value=mocker.Mock(
             **{
+                "status_code": 200,
                 "json.return_value": {
                     "UserDetails": {"Groups": ["tuxsuite"], "Name": "tux"}
-                }
+                },
             }
         )
     )
@@ -65,10 +66,10 @@ def home(monkeypatch, tmp_path):
 
 
 @pytest.fixture
-def config(monkeypatch, sample_token, sample_url):
+def config(monkeypatch, sample_token, sample_url, tuxauth):
     monkeypatch.setenv("TUXSUITE_TOKEN", sample_token)
     monkeypatch.setenv("TUXSUITE_URL", sample_url)
-    config = tuxsuite.config.Config("/dev/null")
+    config = tuxsuite.config.Config("/nonexistent")
     config.kbapi_url = sample_url
     config.auth_token = sample_token
     return config
