@@ -49,7 +49,9 @@ class TestPostRequest:
 
         with pytest.raises(requests.exceptions.HTTPError):
             tuxsuite.build.post_request(
-                url="http://foo.bar.com/timeout", headers=headers, request=request
+                url="http://foo.bar.com/timeout",
+                headers=headers,
+                request=request,
             )
 
     def test_post_request_bad_request(self, post, response):
@@ -71,7 +73,10 @@ class TestGetRequest:
             url="http://foo.bar.com/pass", headers=headers
         ) == {"a": "b"}
         get.assert_called_with(
-            "http://foo.bar.com/pass", headers=headers, timeout=mocker.ANY
+            "http://foo.bar.com/pass",
+            headers=headers,
+            timeout=mocker.ANY,
+            params=None,
         )
 
     def test_get_request_timeout(self, get, response):
@@ -150,7 +155,9 @@ def build_attrs():
 
 @pytest.fixture
 def build(build_attrs):
-    return tuxsuite.build.Build(**build_attrs)
+    b = tuxsuite.build.Build(**build_attrs)
+    b.uid = "myuid"
+    return b
 
 
 class TestBuild:
@@ -208,7 +215,9 @@ class TestBuild:
 
     def test_submit_build_git_ref(self, build, build_attrs, mocker):
         post_request = mocker.patch("tuxsuite.build.post_request")
-        api_build_url = build_attrs["kbapi_url"] + "/build"
+        api_build_url = (
+            build_attrs["tuxapi_url"] + "/v1/groups/tuxsuite/projects/unittests/builds"
+        )
 
         build.build()
         post_request.assert_called_with(
@@ -233,7 +242,9 @@ class TestBuild:
 
     def test_submit_build_git_sha(self, build, build_attrs, mocker):
         post_request = mocker.patch("tuxsuite.build.post_request")
-        api_build_url = build_attrs["kbapi_url"] + "/build"
+        api_build_url = (
+            build_attrs["tuxapi_url"] + "/v1/groups/tuxsuite/projects/unittests/builds"
+        )
 
         build.git_ref = None
         build.git_sha = "badbee"
@@ -269,7 +280,9 @@ class TestBuild:
             "KCONFIG_ALLCONFIG": "arch/arm64/configs/defconfig",
         }
         post_request = mocker.patch("tuxsuite.build.post_request")
-        api_build_url = build_attrs["kbapi_url"] + "/build"
+        api_build_url = (
+            build_attrs["tuxapi_url"] + "/v1/groups/tuxsuite/projects/unittests/builds"
+        )
 
         build.git_ref = None
         build.git_sha = "badbee"
@@ -300,7 +313,9 @@ class TestBuild:
     def test_submit_build_targets(self, build, build_attrs, mocker):
         build_attrs["targets"] = ["dtbs", "config"]
         post_request = mocker.patch("tuxsuite.build.post_request")
-        api_build_url = build_attrs["kbapi_url"] + "/build"
+        api_build_url = (
+            build_attrs["tuxapi_url"] + "/v1/groups/tuxsuite/projects/unittests/builds"
+        )
 
         build.git_ref = None
         build.git_sha = "badbee"
@@ -329,7 +344,9 @@ class TestBuild:
     def test_submit_build_make_variables(self, build, build_attrs, mocker):
         build_attrs["make_variables"] = {"W": "12", "LLVM": "1"}
         post_request = mocker.patch("tuxsuite.build.post_request")
-        api_build_url = build_attrs["kbapi_url"] + "/build"
+        api_build_url = (
+            build_attrs["tuxapi_url"] + "/v1/groups/tuxsuite/projects/unittests/builds"
+        )
 
         build.git_ref = None
         build.git_sha = "badbee"
@@ -375,16 +392,22 @@ class TestBuildWatch(TestWatch):
     def build_statuses(self):
         return [
             {
+                "state": "queued",
+                "result": "unknown",
                 "tuxbuild_status": "queued",
                 "status_message": "Queued",
                 "git_short_log": "Bla bla bla",
             },
             {
+                "state": "building",
+                "result": "unknown",
                 "tuxbuild_status": "building",
                 "status_message": "Building ...",
                 "git_short_log": "Bla bla bla",
             },
             {
+                "state": "finished",
+                "result": "pass",
                 "tuxbuild_status": "complete",
                 "status_message": "Building ...",
                 "git_short_log": "Bla bla bla",
@@ -478,6 +501,8 @@ class TestBuildWatch(TestWatch):
     def test_retries_on_errors(self, build, mocker, build_statuses):
         build_statuses[-1]["build_status"] = None
         build_statuses[-1]["tuxbuild_status"] = "error"
+        build_statuses[-1]["state"] = "finished"
+        build_statuses[-1]["result"] = "error"
         build_statuses[-1]["status_message"] = "the infrastructure failed"
         build_statuses.append(build_statuses[0])
         build_statuses.append(build_statuses[1])
@@ -505,6 +530,8 @@ class TestBuildWatch(TestWatch):
         complete = build_statuses[-1].copy()
         build_statuses[-1]["build_status"] = None
         build_statuses[-1]["tuxbuild_status"] = "error"
+        build_statuses[-1]["state"] = "finished"
+        build_statuses[-1]["result"] = "error"
         build_statuses[-1]["status_message"] = "the infrastructure failed"
         build_statuses.append(build_statuses[0])
         build_statuses.append(build_statuses[1])
@@ -528,7 +555,14 @@ class TestBuildWatch(TestWatch):
         assert [s.state for s in states] == ["queued", "complete"]
 
     def test_timeout(self, build, mocker, get_request):
-        get_request.side_effect = iter(lambda: {"tuxbuild_status": "queued"}, 1)
+        get_request.side_effect = iter(
+            lambda: {
+                "state": "queued",
+                "result": "unknown",
+                "tuxbuild_status": "queued",
+            },
+            1,
+        )
         with pytest.raises(tuxsuite.exceptions.Timeout):
             self.watch(build)
 
@@ -536,6 +570,8 @@ class TestBuildWatch(TestWatch):
         build_statuses.insert(
             2,
             {
+                "state": "spiralling",
+                "result": "spiralling",
                 "tuxbuild_status": "spiralling",
                 "status_message": "Spiralling out of control",
                 "git_short_log": "Bla bla bla",
@@ -609,23 +645,23 @@ class TestBuildSet:
         builds = build_set.builds
         server_data = [
             {
-                "build_key": "00000000000",
+                "uid": "myuid1",
                 "download_url": "https://builds.example/com/00000000000",
             },
             {
-                "build_key": "11111111111",
+                "uid": "myuid2",
                 "download_url": "https://builds.example/com/11111111111",
             },
             {
-                "build_key": "22222222222",
+                "uid": "myuid3",
                 "download_url": "https://builds.example/com/22222222222",
             },
             {
-                "build_key": "33333333333",
+                "uid": "myuid4",
                 "download_url": "https://builds.example/com/33333333333",
             },
             {
-                "build_key": "44444444444",
+                "uid": "myuid5",
                 "download_url": "https://builds.example/com/44444444444",
             },
         ]
@@ -636,11 +672,11 @@ class TestBuildSet:
         response._content = json.dumps(list(reversed(data))).encode()
         build_set.build()
 
-        assert builds[0].build_key == "00000000000"
-        assert builds[1].build_key == "11111111111"
-        assert builds[2].build_key == "22222222222"
-        assert builds[3].build_key == "33333333333"
-        assert builds[4].build_key == "44444444444"
+        assert builds[0].uid == "myuid1"
+        assert builds[1].uid == "myuid2"
+        assert builds[2].uid == "myuid3"
+        assert builds[3].uid == "myuid4"
+        assert builds[4].uid == "myuid5"
 
 
 class TestBuildSetWatch(TestWatch):
