@@ -10,7 +10,7 @@ sample_url = "https://foo.bar.tuxbuild.com/v1"
 
 
 def test_usage():
-    """ Test running cli() with no arguments """
+    """Test running cli() with no arguments"""
     runner = CliRunner()
     result = runner.invoke(tuxsuite.cli.cli, [])
     assert result.exit_code == 0
@@ -19,7 +19,7 @@ def test_usage():
 
 
 def test_build_no_args():
-    """ Test calling build() with no options """
+    """Test calling build() with no options"""
     runner = CliRunner()
     result = runner.invoke(tuxsuite.cli.build, [])
     assert result.exit_code == 2
@@ -28,7 +28,7 @@ def test_build_no_args():
 
 
 def test_build_usage():
-    """ Test calling build() with --help """
+    """Test calling build() with --help"""
     runner = CliRunner()
     result = runner.invoke(tuxsuite.cli.build, ["--help"])
     assert result.exit_code == 0
@@ -233,7 +233,7 @@ def test_build_show_logs(mocker, tuxsuite_config):
 def test_build_download_show_logs(mocker, tuxsuite_config, tmp_path):
     Build = mocker.patch("tuxsuite.Build")
     build = Build.return_value
-    build.build_key = "21321312312"
+    build.uid = "21321312312"
     build.build_data = "https://builds.com/21321312312/"
     wait_for_object = mocker.patch("tuxsuite.cli.wait_for_object")
     runner = CliRunner()
@@ -292,6 +292,27 @@ def tux_config(tmp_path, tuxauth):
     return config
 
 
+sample_plan = """
+version: 1
+name: Simple plan
+description: A simple plan
+jobs:
+
+- builds:
+    - {toolchain: gcc-8, target_arch: i386, kconfig: tinyconfig}
+    - {toolchain: gcc-9, target_arch: i386, kconfig: tinyconfig}
+  test: {device: qemu-i386, tests: [ltp-smoke]}
+"""
+
+
+@pytest.fixture
+def plan_config(tmp_path, tuxauth):
+    config = tmp_path / "plan.yaml"
+    with config.open("w") as f:
+        f.write(sample_plan)
+    return config
+
+
 def test_build_set(mocker, tuxsuite_config, tux_config):
     build = mocker.patch("tuxsuite.BuildSet.build")
     wait_for_object = mocker.patch("tuxsuite.cli.wait_for_object")
@@ -477,7 +498,7 @@ def test_wait_for_object_infra_failure_retried(mocker):
 
 
 def test_build_environment_option():
-    """ Test calling build with --help to see environment option"""
+    """Test calling build with --help to see environment option"""
     runner = CliRunner()
     result = runner.invoke(tuxsuite.cli.build, ["--help"])
     assert result.exit_code == 0
@@ -526,7 +547,7 @@ def test_invalid_environment_key_value(mocker, tuxsuite_config):
 
 
 def test_build_make_targets_vars_argument():
-    """ Test calling build with --help to see targets argument"""
+    """Test calling build with --help to see targets argument"""
     runner = CliRunner()
     result = runner.invoke(tuxsuite.cli.build, ["--help"])
     assert result.exit_code == 0
@@ -535,7 +556,7 @@ def test_build_make_targets_vars_argument():
 
 
 def test_build_set_make_targets_vars_argument():
-    """ Test calling build with --help to see targets argument"""
+    """Test calling build with --help to see targets argument"""
     runner = CliRunner()
     result = runner.invoke(tuxsuite.cli.build_set, ["--help"])
     assert result.exit_code == 0
@@ -604,3 +625,235 @@ def test_build_invalid_make_vars(mocker, tuxsuite_config):
     assert result.exit_code == 1
     assert build.build.call_count == 0
     assert wait_for_object.call_count == 0
+
+
+def test_plan_no_args():
+    runner = CliRunner()
+    result = runner.invoke(tuxsuite.cli.plan, [])
+    assert result.exit_code == 2
+    assert "Usage" in result.output
+    assert "help" in result.output
+
+
+def test_plan_usage():
+    """Test calling plan() with --help"""
+    runner = CliRunner()
+    result = runner.invoke(tuxsuite.cli.plan, ["--help"])
+    assert result.exit_code == 0
+    assert "Usage" in result.output
+    assert "--git-repo" in result.output
+    assert "--name" in result.output
+    assert "--description" in result.output
+
+
+def test_plan_cli(mocker, plan_config):
+    plan = mocker.patch("tuxsuite.Plan")
+    mocker.patch("tuxsuite.cli.wait_for_object")
+    runner = CliRunner()
+    result = runner.invoke(
+        tuxsuite.cli.plan,
+        [
+            "--git-repo=https://git.example.com/linux.git",
+            "--git-ref=master",
+            str(plan_config),
+        ],
+    )
+    assert result.exit_code == 0
+    plan.assert_called_once()
+
+
+def test_plan_cli_options(mocker, plan_config, tuxsuite_config):
+    download = mocker.patch("tuxsuite.download.download")
+    plan_obj = tuxsuite.Plan(
+        "", token="token", tuxapi_url="http://api", project="prj", group="grp"
+    )
+    plan_obj.builds = [1, 2]
+    plan_obj.submit = mocker.Mock()
+    plan = mocker.patch("tuxsuite.Plan")
+    plan.return_value = plan_obj
+
+    show_log = mocker.patch("tuxsuite.cli.show_log")
+    wait_for_object = mocker.patch("tuxsuite.cli.wait_for_object")
+    runner = CliRunner()
+    result = runner.invoke(
+        tuxsuite.cli.plan,
+        [
+            "--git-repo=https://git.example.com/linux.git",
+            "--git-ref=master",
+            "--download",
+            "--show-logs",
+            str(plan_config),
+        ],
+    )
+    print(result.output)
+    assert result.exit_code == 0
+    plan.assert_called_once()
+    assert wait_for_object.call_count == 1
+    assert download.call_count == 2
+    assert show_log.call_count == 2
+
+
+def test_test_no_args():
+    runner = CliRunner()
+    result = runner.invoke(tuxsuite.cli.test, [])
+    assert result.exit_code == 2
+    assert "Usage" in result.output
+    assert "help" in result.output
+
+
+def test_test_usage():
+    """Test calling test() with --help"""
+    runner = CliRunner()
+    result = runner.invoke(tuxsuite.cli.test, ["--help"])
+    assert result.exit_code == 0
+    assert "Usage" in result.output
+    assert "--device" in result.output
+    assert "--kernel" in result.output
+
+
+def test_test_cli(mocker, tuxsuite_config):
+    test = mocker.patch("tuxsuite.Test")
+    wait_for_object = mocker.patch("tuxsuite.cli.wait_for_object")
+    runner = CliRunner()
+    result = runner.invoke(
+        tuxsuite.cli.test,
+        [
+            "--device=qemu-arm64",
+            "--kernel=https://example.com/vmlinux",
+        ],
+    )
+    assert result.exit_code == 0
+    assert test.call_count == 1
+    test.assert_called_once_with(
+        device="qemu-arm64",
+        kernel="https://example.com/vmlinux",
+        modules=None,
+        tests=[],
+        wait_for=None,
+    )
+    assert wait_for_object.call_count == 1
+
+    mocker.resetall()
+    result = runner.invoke(
+        tuxsuite.cli.test,
+        [
+            "--device=qemu-arm64",
+            "--kernel=https://example.com/vmlinux",
+            "--modules=https://example.com/modules.tar.xz",
+            "--tests=ltp-smoke",
+        ],
+    )
+    assert result.exit_code == 0
+    assert test.call_count == 1
+    test.assert_called_once_with(
+        device="qemu-arm64",
+        kernel="https://example.com/vmlinux",
+        modules="https://example.com/modules.tar.xz",
+        tests=["ltp-smoke"],
+        wait_for=None,
+    )
+    assert wait_for_object.call_count == 1
+
+    mocker.resetall()
+    result = runner.invoke(
+        tuxsuite.cli.test,
+        [
+            "--device=qemu-arm64",
+            "--tests=ltp-smoke",
+            "--wait-for=mybuilduid",
+        ],
+    )
+    assert result.exit_code == 0
+    assert test.call_count == 1
+    test.assert_called_once_with(
+        device="qemu-arm64",
+        kernel="",
+        modules=None,
+        tests=["ltp-smoke"],
+        wait_for="mybuilduid",
+    )
+    assert wait_for_object.call_count == 1
+
+
+def test_test_cli_errors(mocker):
+    test = mocker.patch("tuxsuite.Test")
+    wait_for_object = mocker.patch("tuxsuite.cli.wait_for_object")
+    runner = CliRunner()
+    result = runner.invoke(
+        tuxsuite.cli.test,
+        [
+            "--device=qemu-arm64",
+            "--tests=something,ltp-smoke",
+        ],
+    )
+    assert result.exit_code == 1
+    assert test.call_count == 0
+    assert wait_for_object.call_count == 0
+
+    mocker.resetall()
+    result = runner.invoke(
+        tuxsuite.cli.test,
+        [
+            "--device=qemu-arm64",
+            "--kernel=https://example.com/vmlinux",
+            "--wait-for=mybuilduid",
+        ],
+    )
+    assert result.exit_code == 1
+    assert test.call_count == 0
+
+
+def test_build_no_wait(mocker):
+    build = mocker.patch("tuxsuite.Build.build")
+    wait_for_object = mocker.patch("tuxsuite.cli.wait_for_object")
+    runner = CliRunner()
+    result = runner.invoke(
+        tuxsuite.cli.build,
+        [
+            "--git-repo=https://git.example.com/linux.git",
+            "--git-ref=master",
+            "--target-arch=arm64",
+            "--kconfig=defconfig",
+            "--toolchain=gcc-9",
+            "--no-wait",
+        ],
+    )
+    assert result.exit_code == 0
+    assert build.call_count == 1
+    assert wait_for_object.call_count == 0
+
+
+def test_build_set_no_wait(mocker, tuxsuite_config, tux_config):
+    build = mocker.patch("tuxsuite.BuildSet.build")
+    wait_for_object = mocker.patch("tuxsuite.cli.wait_for_object")
+    runner = CliRunner()
+    result = runner.invoke(
+        tuxsuite.cli.build_set,
+        [
+            "--git-repo=https://git.example.com/linux.git",
+            "--git-ref=master",
+            f"--tux-config={tux_config}",
+            "--set-name=test",
+            "--no-wait",
+        ],
+    )
+    assert result.exit_code == 0
+    assert build.call_count == 1
+    assert wait_for_object.call_count == 0
+
+
+def test_test_cli_no_wait(mocker, tuxsuite_config):
+    test = mocker.patch("tuxsuite.Test")
+    wait_for_object = mocker.patch("tuxsuite.cli.wait_for_object")
+    runner = CliRunner()
+    result = runner.invoke(
+        tuxsuite.cli.test,
+        [
+            "--device=qemu-arm64",
+            "--kernel=https://example.com/vmlinux",
+            "--no-wait",
+        ],
+    )
+    assert result.exit_code == 0
+    assert test.call_count == 1
+    assert wait_for_object.call_count == 0
