@@ -48,6 +48,12 @@ class BuildSet(Configurable, build.BuildSet):
     """
 
 
+class Plan(Configurable, build.Plan):
+    """
+    This class represent a test plan.
+    """
+
+
 class Test(Configurable, build.Test):
     """
     This class represents individual tests. It should be used to trigger
