@@ -27,16 +27,21 @@ def post_request(url, headers, request):
         return json.loads(response.text)
     else:
         if response.status_code == 400:
-            response_data = json.loads(response.text)
-            raise tuxsuite.exceptions.BadRequest(
-                f"{response_data.get('status_message')}"
-            )
+            response_data = response.json()
+            if "error" in response_data:
+                message = response_data["error"]
+            else:
+                message = response_data.get("status_message")
+            raise tuxsuite.exceptions.BadRequest(message)
         else:
-            response.raise_for_status()
+            try:
+                raise Exception(response.json()["error"])
+            except Exception:
+                response.raise_for_status()
 
 
-def get_request(url, headers):
-    response = tuxsuite.requests.get(url, headers=headers)
+def get_request(url, headers, params=None):
+    response = tuxsuite.requests.get(url, headers=headers, params=params)
     if response.status_code == 200:
         return json.loads(response.text)
     else:
@@ -85,9 +90,9 @@ class Build(HeadersMixin):
     kernel_image = attrib(default=None)
 
     build_data = attrib(default=None)
-    build_key = attrib(default=None)
     build_name = attrib(default=None)
     status = attrib(default={})
+    uid = attrib(default=None)
 
     def __attrs_post_init__(self):
         if isinstance(self.kconfig, str):
@@ -118,8 +123,8 @@ class Build(HeadersMixin):
         """
         return re.match(r"^(git://|https?://).*$", url) is not None
 
-    def generate_build_request(self):
-        """ return a build data in a python dict """
+    def generate_build_request(self, plan=None):
+        """return a build data in a python dict"""
         build_entry = {}
         build_entry["client_token"] = self.client_token
         build_entry["git_repo"] = self.git_repo
@@ -137,10 +142,12 @@ class Build(HeadersMixin):
             build_entry["kernel_image"] = self.kernel_image
         if self.build_name:
             build_entry["build_name"] = self.build_name
+        if plan is not None:
+            build_entry["plan"] = plan
         return build_entry
 
     def verify_build_parameters(self):
-        """ Pre-check the build arguments """
+        """Pre-check the build arguments"""
         assert self.git_repo, "git_repo is mandatory"
         assert self.is_supported_git_url(
             self.git_repo
@@ -154,20 +161,24 @@ class Build(HeadersMixin):
         assert self.headers is not None, "headers is mandatory"
 
     def build(self):
-        """ Submit the build request """
+        """Submit the build request"""
         data = []
         data.append(self.generate_build_request())
-        url = self.kbapi_url + "/build"
+        url = self.tuxapi_url + "/v1/groups/{}/projects/{}/builds".format(
+            self.group, self.project
+        )
         json_data = post_request(url, self.headers, data)
         self.build_data = json_data[0]["download_url"]
-        self.build_key = json_data[0]["build_key"]
+        self.uid = json_data[0]["uid"]
 
     def get_status(self):
-        """ Fetches the build status and updates the values inside the build object"""
-        url = self.kbapi_url + "/status/" + self.build_key
+        """Fetches the build status and updates the values inside the build object"""
+        url = self.tuxapi_url + "/v1/groups/{}/projects/{}/builds/{}".format(
+            self.group, self.project, self.uid
+        )
         self.status = get_request(url, self.headers)
 
-    def watch(self):
+    def watch(self, iterator=False):
         """
         Wait for the build to finish. Whenever the state changes, yielding the
         new state).
@@ -198,17 +209,28 @@ class Build(HeadersMixin):
                 icon="🚀",
                 cli_color="cyan",
             ),
+            "running": BuildState(
+                build=self,
+                state="running",
+                message="Running",
+                icon="🚀",
+                cli_color="cyan",
+            ),
         }
-        final_states = ["complete", "error"]
         timeout = time.time() + state_timeout
         previous_state = None
         while True:
             self.get_status()
-            state = self.status["tuxbuild_status"]
+            state = self.status["state"]
+            result = self.status["result"]
             if state != previous_state:
                 if state in waiting_states:
                     yield waiting_states[state]
-                elif state == "error" and attempts < max_attempts:
+                elif (
+                    state == "finished"
+                    and result == "error"
+                    and attempts < max_attempts
+                ):
                     attempts += 1
                     message = self.status.get("status_message", "infrastructure error")
                     yield BuildState(
@@ -220,7 +242,7 @@ class Build(HeadersMixin):
                         cli_color="yellow",
                     )
                     self.build()
-                elif state in final_states:
+                elif state == "finished":
                     break
                 else:
                     # unknown state
@@ -235,8 +257,12 @@ class Build(HeadersMixin):
                 raise tuxsuite.exceptions.Timeout(
                     f"Build timed out after {state_timeout/60} minutes; abandoning"
                 )
-            delay = random.randrange(delay_status_update)
-            time.sleep(delay)
+            elif iterator:
+                yield
+
+            if not iterator:
+                delay = random.randrange(delay_status_update)
+                time.sleep(delay)
             previous_state = state
 
         errors = 0
@@ -293,33 +319,33 @@ class Build(HeadersMixin):
         return state
 
     def _get_field(self, field):
-        """ Retrieve an individual field from status.json """
+        """Retrieve an individual field from status.json"""
         self.get_status()
         return self.status.get(field, None)
 
     @property
     def warnings_count(self):
-        """ Get the warnings_count for the build """
+        """Get the warnings_count for the build"""
         return int(self._get_field("warnings_count"))
 
     @property
     def errors_count(self):
-        """ Get the errors_count for the build """
+        """Get the errors_count for the build"""
         return int(self._get_field("errors_count"))
 
     @property
     def tuxbuild_status(self):
-        """ Get the tuxbuild_status for the build """
-        return self._get_field("tuxbuild_status")
+        """Get the tuxbuild_status for the build"""
+        return self._get_field("state")
 
     @property
     def build_status(self):
-        """ Get the build_status for the build """
-        return self._get_field("build_status")
+        """Get the build_status for the build"""
+        return self._get_field("result")
 
     @property
     def status_message(self):
-        """ Get the build_status for the build """
+        """Get the build_status for the build"""
         return self._get_field("status_message")
 
 
@@ -331,13 +357,17 @@ class BuildSet(HeadersMixin):
             data.update(item)
             self.builds.append(Build(**data))
         self.token = kwargs["token"]
-        self.kbapi_url = kwargs["kbapi_url"]
+        self.tuxapi_url = kwargs["tuxapi_url"]
+        self.project = kwargs["project"]
+        self.group = kwargs["group"]
 
     def build(self):
-        """ Submit the build request """
+        """Submit the build request"""
 
         data = []
-        url = self.kbapi_url + "/build"
+        url = self.tuxapi_url + "/v1/groups/{}/projects/{}/builds".format(
+            self.group, self.project
+        )
         for build_object in self.builds:
             data.append(build_object.generate_build_request())
         json_data = post_request(url, self.headers, data)
@@ -346,7 +376,7 @@ class BuildSet(HeadersMixin):
         for build_config in json_data:
             build = builds_by_client_token[build_config["client_token"]]
             build.build_data = build_config["download_url"]
-            build.build_key = build_config["build_key"]
+            build.uid = build_config["uid"]
 
     def __watch_build__(self, build, results):
         for state in build.watch():
@@ -360,7 +390,7 @@ class BuildSet(HeadersMixin):
             futures = {}
             for build in self.builds:
                 f = executor.submit(self.__watch_build__, build, results)
-                futures[build.build_key] = f
+                futures[build.uid] = f
 
             finished = 0
             while finished < n:
@@ -394,10 +424,161 @@ class BuildSet(HeadersMixin):
 
     @property
     def status_list(self):
-        """ Return a list of build status dictionaries """
+        """Return a list of build status dictionaries"""
         return [build.status for build in self.builds]
 
 
+class Plan(HeadersMixin):
+    def __init__(self, config, **kwargs):
+        self.config = config
+        self.token = kwargs["token"]
+        self.tuxapi_url = kwargs["tuxapi_url"]
+        self.project = kwargs["project"]
+        self.group = kwargs["group"]
+        self.args = kwargs
+        self.plan = None
+        self.builds = []
+        self.tests = []
+        self.url = self.tuxapi_url + "/v1/groups/{}/projects/{}".format(
+            self.group, self.project
+        )
+
+    def submit(self):
+        builds = []
+        tests = []
+
+        # Create the plan
+        self.plan = post_request(
+            f"{self.url}/plans",
+            self.headers,
+            {"name": self.config.name, "description": self.config.description},
+        )["uid"]
+
+        # Create the builds
+        for cfg in self.config.plan:
+            if cfg["build"] is not None:
+                data = self.args.copy()
+                data.update(cfg["build"])
+                builds.append(Build(**data))
+            else:
+                builds.append(None)
+
+        builds_to_submit = [b for b in builds if b]
+        if builds_to_submit:
+            ret = post_request(
+                f"{self.url}/builds",
+                self.headers,
+                [b.generate_build_request(plan=self.plan) for b in builds_to_submit],
+            )
+            # Updating builds_to_submit will update values in builds
+            for (build, data) in zip(builds_to_submit, ret):
+                build.build_data = data["download_url"]
+                build.uid = data["uid"]
+
+        # Create the tests
+        for (build, cfg) in zip(builds, self.config.plan):
+            if build:
+                for test in cfg["tests"]:
+                    tests.append(
+                        Test(
+                            token=self.token,
+                            kbapi_url="",
+                            tuxapi_url=self.tuxapi_url,
+                            group=self.group,
+                            project=self.project,
+                            device=test["device"],
+                            kernel="",
+                            tests=test.get("tests", []),
+                            wait_for=build.uid,
+                        )
+                    )
+                self.builds.append(build)
+            else:
+                for test in cfg["tests"]:
+                    tests.append(
+                        Test(
+                            token=self.token,
+                            kbapi_url="",
+                            tuxapi_url=self.tuxapi_url,
+                            group=self.group,
+                            project=self.project,
+                            device=test["device"],
+                            kernel=test["kernel"],
+                            modules=test.get("modules"),
+                            tests=test.get("tests", []),
+                        )
+                    )
+
+        ret = post_request(
+            f"{self.url}/tests",
+            self.headers,
+            [t.generate_test_request(plan=self.plan) for t in tests],
+        )
+        for (test, data) in zip(tests, ret):
+            test.uid = data["uid"]
+            self.tests.append(test)
+
+    def get_plan(self):
+        builds = {}
+        tests = {}
+        first = True
+        start_builds = start_tests = None
+        while first or start_builds is not None or start_tests is not None:
+            ret = get_request(
+                f"{self.url}/plans/{self.plan}",
+                headers=self.headers,
+                params={"start_builds": start_builds, "start_tests": start_tests},
+            )
+            if first or start_builds is not None:
+                builds.update({b["uid"]: b for b in ret["builds"]["results"]})
+            if first or start_tests is not None:
+                tests.update({t["uid"]: t for t in ret["tests"]["results"]})
+
+            first = False
+            start_builds = ret["builds"]["next"]
+            start_tests = ret["tests"]["next"]
+
+        return {"builds": builds, "tests": tests}
+
+    def watch(self):
+        finished = {"builds": [], "tests": []}
+        while True:
+            plan = self.get_plan()
+            for build in self.builds:
+                if build.uid not in finished["builds"]:
+                    build.status = plan["builds"][build.uid]
+                    if not hasattr(build, "ite"):
+                        build.get_status = lambda: None
+                        build.ite = build.watch(iterator=True)
+                    state = next(build.ite)
+                    if state is not None:
+                        yield state
+                        if state.final:
+                            finished["builds"].append(build.uid)
+            for test in self.tests:
+                if test.uid not in finished["tests"]:
+                    test.get = lambda: plan["tests"][test.uid]
+                    if not hasattr(test, "ite"):
+                        test.ite = test.watch(iterator=True)
+                    state = next(test.ite)
+                    if state is not None:
+                        yield state
+                        if state.final:
+                            finished["tests"].append(test.uid)
+            if len(finished["builds"]) == len(self.builds) and len(
+                finished["tests"]
+            ) == len(self.tests):
+                break
+            time.sleep(5)
+
+    def wait(self):
+        states = []
+        for s in self.watch():
+            if s.final:
+                states.append(s)
+        return states
+
+
 @attrs(kw_only=True)
 class Test(HeadersMixin):
     token = attrib()
@@ -407,7 +588,9 @@ class Test(HeadersMixin):
     project = attrib()
     device = attrib()
     kernel = attrib()
+    modules = attrib(default=None)
     tests = attrib(default=[])
+    wait_for = attrib(default=None)
     uid = attrib(default=None)
 
     def __str__(self):
@@ -417,13 +600,24 @@ class Test(HeadersMixin):
         )
         return "{} {} @ {}".format(tests, self.device, url)
 
-    def test(self):
+    def generate_test_request(self, plan=None):
+        data = {"device": self.device, "kernel": self.kernel}
+        if self.modules:
+            data["modules"] = self.modules
+        if self.tests:
+            data["tests"] = self.tests
+        if self.wait_for:
+            data["waiting_for"] = self.wait_for
+        if plan is not None:
+            data["plan"] = plan
+
+        return data
+
+    def test(self, plan=None):
         url = self.tuxapi_url + "/v1/groups/{}/projects/{}/tests".format(
             self.group, self.project
         )
-        data = {"device": self.device, "kernel": self.kernel}
-        if self.tests:
-            data["tests"] = self.tests
+        data = self.generate_test_request(plan=plan)
         data = post_request(url, self.headers, data)
         self.uid = data["uid"]
 
@@ -433,8 +627,15 @@ class Test(HeadersMixin):
         )
         return get_request(url, self.headers)
 
-    def watch(self):
+    def watch(self, iterator=False):
         waiting_states = {
+            "waiting": BuildState(
+                build=self,
+                state="waiting",
+                message="Waiting",
+                icon="⏳",
+                cli_color="white",
+            ),
             "provisioning": BuildState(
                 build=self,
                 state="provisioning",
@@ -462,8 +663,12 @@ class Test(HeadersMixin):
                     break
                 else:
                     assert 0
-            delay = random.randrange(delay_status_update)
-            time.sleep(delay)
+            elif iterator:
+                yield
+
+            if not iterator:
+                delay = random.randrange(delay_status_update)
+                time.sleep(delay)
             previous_state = state
 
         errors = 0
@@ -496,3 +701,9 @@ class Test(HeadersMixin):
             errors=errors,
         )
         yield finished
+
+    def wait(self):
+        state = None
+        for s in self.watch():
+            state = s
+        return state
