@@ -112,7 +112,7 @@ def common_options(required):
         ),
         option(
             "--toolchain",
-            help="Toolchain [gcc-8|gcc-9|gcc-10|clang-10|clang-11|clang-12|clang-nightly]",
+            help="Toolchain [gcc-8|gcc-9|gcc-10|clang-10|clang-11|clang-12|clang-nightly|clang-android]",
         ),
         option(
             "--build-name",
@@ -136,6 +136,13 @@ def common_options(required):
             default=".",
             help="Directory where to download artifacts",
         ),
+        option(
+            "-n",
+            "--no-wait",
+            default=False,
+            is_flag=True,
+            help="Don't wait for the builds to finish",
+        ),
         option(
             "-q",
             "--quiet",
@@ -189,7 +196,7 @@ def show_log(build, download, output_dir):
     print("📄 Logs for {}:".format(build), file=sys.stderr)
     sys.stderr.flush()
     if download:
-        for line in open(os.path.join(output_dir, build.build_key, "build.log")):
+        for line in open(os.path.join(output_dir, build.uid, "build.log")):
             print(line.strip(), file=sys.stderr)
     else:
         tuxsuite.download.download_file(
@@ -216,6 +223,7 @@ def build(
     git_head=False,
     download=False,
     output_dir=None,
+    no_wait=False,
     **build_params,
 ):
     quiet_output(quiet)
@@ -238,6 +246,9 @@ def build(
         build.build()
     except tuxsuite.exceptions.BadRequest as e:
         raise (click.ClickException(str(e)))
+    if no_wait:
+        return
+
     build_result = wait_for_object(build)
     if json_out:
         json_out.write(json.dumps(build.status, sort_keys=True, indent=4))
@@ -265,6 +276,7 @@ def build_set(
     git_head=False,
     download=False,
     output_dir=None,
+    no_wait=False,
     **build_params,
 ):
     quiet_output(quiet)
@@ -286,6 +298,8 @@ def build_set(
         build_set.build()
     except tuxsuite.exceptions.BadRequest as e:
         raise (click.ClickException(str(e)))
+    if no_wait:
+        return
 
     build_set_result = wait_for_object(build_set)
 
@@ -311,21 +325,41 @@ def build_set(
 
 DEVICES = [
     "qemu-arm",
+    "qemu-armv5",
+    "qemu-armv7",
     "qemu-arm64",
     "qemu-i386",
+    "qemu-mips",
     "qemu-mips64",
+    "qemu-mips64el",
+    "qemu-powerpc",
     "qemu-ppc64",
+    "qemu-riscv",
     "qemu-riscv64",
+    "qemu-sparc64",
     "qemu-x86_64",
 ]
-TESTS = ["ltp-smoke"]
+TESTS = ["boot", "ltp-smoke"]
 
 
 @cli.command(help="Test a kernel", short_help="Test a kernel")
 @click.option("--device", help="Device type", required=True, type=click.Choice(DEVICES))
-@click.option("--kernel", help="URL of the kernel to test", required=True, type=str)
-@click.option("--tests", help="Comma separated list of tests", type=str, default="")
-def test(device, kernel, tests):
+@click.option("--kernel", help="URL of the kernel to test", default="", type=str)
+@click.option("--modules", help="URL of the kernel modules", default=None, type=str)
+@click.option(
+    "--tests",
+    help="Comma separated list of tests",
+    default="boot",
+)
+@click.option("--wait-for", help="Wait for a test uid", default=None, type=str)
+@click.option(
+    "-n",
+    "--no-wait",
+    default=False,
+    is_flag=True,
+    help="Don't wait for tests to finish",
+)
+def test(device, kernel, modules, tests, wait_for, no_wait):
     tests = [test for test in tests.split(",") if test]
     invalid_tests = [test for test in tests if test not in TESTS]
     if invalid_tests:
@@ -334,10 +368,34 @@ def test(device, kernel, tests):
                 ", ".join(invalid_tests), ", ".join(TESTS)
             )
         )
-    info("Testing {} on {} with {}".format(kernel, device, ", ".join(["boot"] + tests)))
+    tests = [test for test in tests if test != "boot"]
+    if wait_for:
+        info(
+            "Testing build {} on {} with {}".format(
+                wait_for, device, ", ".join(["boot"] + tests)
+            )
+        )
+        if kernel:
+            raise click.ClickException("--kernel and --wait-for are mutually exclusive")
+        if modules:
+            raise click.ClickException(
+                "--modules and --wait-for are mutually exclusive"
+            )
+    else:
+        info(
+            "Testing {} on {} with {}".format(
+                kernel, device, ", ".join(["boot"] + tests)
+            )
+        )
 
     try:
-        test = tuxsuite.Test(device=device, kernel=kernel, tests=tests)
+        test = tuxsuite.Test(
+            device=device,
+            kernel=kernel,
+            modules=modules,
+            tests=tests,
+            wait_for=wait_for,
+        )
     except (AssertionError, tuxsuite.exceptions.TuxSuiteError) as e:
         error(e)
 
@@ -345,6 +403,9 @@ def test(device, kernel, tests):
         test.test()
     except tuxsuite.exceptions.BadRequest as e:
         raise (click.ClickException(str(e)))
+    if no_wait:
+        return
+
     test_result = wait_for_object(test)
 
     # If the test did not pass, exit with exit code of 1
@@ -352,5 +413,88 @@ def test(device, kernel, tests):
         sys.exit(1)
 
 
+@cli.command(help="Run the specified plan file", short_help="Run a plan file.")
+@click.option("--name", default=None, help="Set name")
+@click.option("--description", default=None, help="Set description")
+@click.option("--git-repo", default=None, help="Git repository")
+@click.option("--git-ref", default=None, help="Git reference")
+@click.option("--git-sha", default=None, help="Git commit")
+@click.option(
+    "--git-head",
+    default=False,
+    is_flag=True,
+    help="Build the current git HEAD. Overrrides --git-repo and --git-ref",
+)
+@click.option(
+    "-d",
+    "--download",
+    default=False,
+    is_flag=True,
+    help="Download artifacts after builds finish",
+)
+@click.option(
+    "-o",
+    "--output-dir",
+    default=".",
+    help="Directory where to download artifacts",
+)
+@click.option(
+    "-s",
+    "--show-logs",
+    default=False,
+    is_flag=True,
+    help="Prints build logs to stderr in case of warnings or errors",
+)
+@click.argument("config")
+def plan(
+    name,
+    description,
+    config,
+    show_logs=None,
+    download=False,
+    output_dir=None,
+    **build_params,
+):
+    if build_params["git_head"]:
+        try:
+            repo, sha = tuxsuite.gitutils.get_git_head()
+            build_params["git_repo"] = repo
+            build_params["git_sha"] = sha
+        except Exception as e:
+            error(e)
+    del build_params["git_head"]
+
+    try:
+        plan_config = tuxsuite.config.PlanConfig(name, description, config)
+        plan = tuxsuite.Plan(plan_config, **build_params)
+    except (AssertionError, tuxsuite.exceptions.TuxSuiteError) as e:
+        error(e)
+
+    info(
+        "Running Linux Kernel plan '{}': '{}'".format(
+            plan_config.name, plan_config.description
+        )
+    )
+
+    try:
+        plan.submit()
+        info("Plan {}/plans/{}".format(plan.url, plan.plan))
+    except tuxsuite.exceptions.BadRequest as e:
+        raise (click.ClickException(str(e)))
+
+    result = wait_for_object(plan)
+
+    if download:
+        for build in plan.builds:
+            tuxsuite.download.download(build, output_dir)
+
+    if show_logs:
+        for build in plan.builds:
+            show_log(build, download, output_dir)
+
+    if not result:
+        sys.exit(1)
+
+
 def main():
     cli.main(prog_name="tuxsuite")
