@@ -33,6 +33,10 @@ def get_config_file(name):
     return config_path
 
 
+def get_defaults_ini(path):
+    return os.path.abspath(os.path.join(expanduser(path), "..", "defaults.ini"))
+
+
 class Config:
     def __init__(self, config_path=None):
         """
@@ -72,35 +76,40 @@ class Config:
                 tuxauth_url (optional)
         """
 
-        self.default_api_url = (
-            "https://api.tuxbuild.com/v1"  # Use production v1 if no url is specified
-        )
+        # defaults
+        self.auth_token = None
+        self.group = None
+        self.project = None
+        self.kbapi_url = "https://api.tuxbuild.com/v1"
         self.tuxapi_url = "https://tuxapi.tuxsuite.com"
         self.tuxauth_url = "https://auth.tuxsuite.com"
+
+        # tuxsuite environment
         self.tuxsuite_env = getenv("ENV", "default")
 
-        (
-            self.auth_token,
-            self.kbapi_url,
-            self.group,
-            self.project,
-        ) = self._get_config_from_env()
-
+        # Select the config file
         config_path = getenv("CONFIG", config_path)
-
         if config_path is None:
             config_path = get_config_file("config.ini")
+        config_path_exists = os.path.exists(expanduser(config_path))
 
-        if not self.auth_token:
-            (
-                self.auth_token,
-                self.kbapi_url,
-                self.tuxapi_url,
-                self.tuxauth_url,
-                self.group,
-                self.project,
-            ) = self._get_config_from_config(config_path)
+        # Load configuration from file
+        if config_path_exists:
+            config = self._get_config_from_file(config_path, self.tuxsuite_env)
+            self.auth_token = config.get("token")
+            self.group = config.get("group")
+            self.project = config.get("project")
+            self.kbapi_url = config.get("api_url", self.kbapi_url).rstrip("/")
+            self.tuxapi_url = config.get("tuxapi_url", self.tuxapi_url).rstrip("/")
+            self.tuxauth_url = config.get("tuxauth_url", self.tuxauth_url).rstrip("/")
 
+        # override with configuration from env variables
+        self.auth_token = getenv("TOKEN", self.auth_token)
+        self.group = getenv("GROUP", self.group)
+        self.project = getenv("PROJECT", self.project)
+        self.kbapi_url = getenv("URL", self.kbapi_url).rstrip("/")
+
+        # token and kbapi should be specified
         if not self.auth_token:
             raise tuxsuite.exceptions.TokenNotFound(
                 "Token not found in TUXSUITE_TOKEN nor at [{}] in {}".format(
@@ -114,76 +123,79 @@ class Config:
                 )
             )
 
-    def _get_config_from_config(self, config_path):
+        # group and project should be defined
+        if self.group is None or self.project is None:
+            (self.group, self.project) = self._get_defaults(
+                config_path,
+                config_path_exists,
+                self.tuxsuite_env,
+                self.group,
+                self.project,
+            )
+
+    def _get_config_from_file(self, config_path, env):
         path = expanduser(config_path)
-        defaults = os.path.abspath(os.path.join(path, "..", "defaults.ini"))
+        defaults = get_defaults_ini(config_path)
+
         try:
-            open(path, "r")  # ensure file exists and is readable
+            with open(path, "r"):
+                # ensure file exists and is readable
+                pass
         except Exception as e:
             raise tuxsuite.exceptions.CantGetConfiguration(str(e))
 
-        config = configparser.ConfigParser()
-
         # Load the defaults config (if it exists) and path
-        config.read([defaults, path])
-        if not config.has_section(self.tuxsuite_env):
+        try:
+            config = configparser.ConfigParser()
+            config.read([defaults, path])
+        except configparser.Error as exc:
             raise tuxsuite.exceptions.InvalidConfiguration(
-                "Error, missing section [{}] from config file '{}'".format(
-                    self.tuxsuite_env, path
-                )
+                "Error, invalid config file '{}': {}".format(path, str(exc))
             )
-        kbapi_url = (
-            config[self.tuxsuite_env].get("api_url", self.default_api_url).rstrip("/")
-        )
-        tuxapi_url = (
-            config[self.tuxsuite_env].get("tuxapi_url", self.tuxapi_url).rstrip("/")
-        )
-        tuxauth_url = (
-            config[self.tuxsuite_env].get("tuxauth_url", self.tuxauth_url).rstrip("/")
-        )
-        auth_token = config[self.tuxsuite_env].get("token", None)
-        group = config[self.tuxsuite_env].get("group", None)
-        project = config[self.tuxsuite_env].get("project", None)
+        if not config.has_section(env):
+            raise tuxsuite.exceptions.InvalidConfiguration(
+                "Error, missing section [{}] from config file '{}'".format(env, path)
+            )
+        return config[env]
+
+    def _get_defaults(self, config_path, config_path_exists, env, group, project):
+        defaults = get_defaults_ini(config_path)
 
         # Add default group and project from tuxauth
-        if auth_token and (group is None or project is None):
-            ret = requests.get(
-                f"{tuxauth_url}/v1/tokens/{uuid.uuid3(uuid.NAMESPACE_DNS, auth_token)}"
-            )
-            try:
-                user = ret.json()["UserDetails"]
-            except Exception as e:
-                raise tuxsuite.exceptions.CantGetConfiguration(
-                    f"Unable to get default group and project: {e}"
+        ret = requests.get(
+            f"{self.tuxauth_url}/v1/tokens/{uuid.uuid3(uuid.NAMESPACE_DNS, self.auth_token)}"
+        )
+        try:
+            if ret.status_code != 200:
+                raise Exception(
+                    f"Unable to authenticate to {self.tuxauth_url}: {ret.status_code}"
                 )
+            user = ret.json()["UserDetails"]
+        except Exception as e:
+            raise tuxsuite.exceptions.CantGetConfiguration(
+                f"Unable to get default group and project: {e}"
+            )
 
+        if config_path_exists:
             default_config = configparser.ConfigParser()
             default_config.read(defaults)
-            if not default_config.has_section(self.tuxsuite_env):
-                default_config.add_section(self.tuxsuite_env)
-            if group is None:
-                group = user["Groups"][0]
-                default_config.set(self.tuxsuite_env, "group", group)
-            if project is None:
-                project = user["Name"]
-                default_config.set(self.tuxsuite_env, "project", project)
+            if not default_config.has_section(env):
+                default_config.add_section(env)
 
+        if group is None:
+            group = user["Groups"][0]
+            if config_path_exists:
+                default_config.set(env, "group", group)
+        if project is None:
+            project = user["Name"]
+            if config_path_exists:
+                default_config.set(env, "project", project)
+
+        if config_path_exists:
             with open(defaults, "w") as f_out:
                 default_config.write(f_out)
-        return (auth_token, kbapi_url, tuxapi_url, tuxauth_url, group, project)
 
-    def _get_config_from_env(self):
-        # Check environment for TUXSUITE_TOKEN
-        auth_token = None
-        kbapi_url = None
-        group = None
-        project = None
-        if getenv("TOKEN"):
-            auth_token = getenv("TOKEN")
-            kbapi_url = getenv("URL", self.default_api_url).rstrip("/")
-            group = getenv("GROUP")
-            project = getenv("PROJECT")
-        return (auth_token, kbapi_url, group, project)
+        return (group, project)
 
     def get_auth_token(self):
         return self.auth_token
@@ -247,3 +259,61 @@ class BuildSetConfig:
                 f"Unable to retrieve {url}: {result.status_code} {result.reason}"
             )
         return result.text
+
+
+class PlanConfig:
+    def __init__(self, name, description, filename):
+        self.name = name
+        self.description = description
+        self.filename = filename
+        self.plan = []
+        self.__load_config__()
+
+    def __load_config__(self):
+        try:
+            if re.match(r"^https?://", str(self.filename)):
+                contents = self.__fetch_remote_config__(self.filename)
+            else:
+                contents = open(self.filename).read()
+            config = yaml.safe_load(contents)
+        except (FileNotFoundError, yaml.loader.ParserError) as e:
+            raise InvalidConfiguration(str(e))
+
+        if not isinstance(config, dict):
+            raise InvalidConfiguration(
+                f"Plan configuration in {self.filename} is invalid"
+            )
+
+        if config.get("version") != 1:
+            raise InvalidConfiguration(f"Invalid plan {self.filename}: invalid version")
+
+        if not self.name:
+            self.name = config["name"]
+        if not self.description:
+            self.description = config["description"]
+
+        for cfg in config["jobs"]:
+            builds = []
+            if "build" in cfg:
+                builds = [cfg["build"]]
+            elif "builds" in cfg:
+                builds = cfg["builds"]
+            tests = []
+            if "test" in cfg:
+                tests = [cfg["test"]]
+            elif "tests" in cfg:
+                tests = cfg["tests"]
+
+            if builds:
+                for build in builds:
+                    self.plan.append({"build": build, "tests": tests})
+            else:
+                self.plan.append({"build": None, "tests": tests})
+
+    def __fetch_remote_config__(self, url):
+        result = requests.get(url)
+        if result.status_code != 200:
+            raise InvalidConfiguration(
+                f"Unable to retrieve {url}: {result.status_code} {result.reason}"
+            )
+        return result.text
