@@ -5,6 +5,9 @@ from requests.adapters import HTTPAdapter
 from requests.packages.urllib3.util.retry import Retry
 
 
+timeout = 60
+
+
 def get_session(*, retries):
     session = requests.Session()
     retry_strategy = Retry(
@@ -21,9 +24,9 @@ def get_session(*, retries):
 
 def get(*args, **kwargs):
     session = get_session(retries=8)
-    return session.get(*args, **kwargs)
+    return session.get(*args, timeout=timeout, **kwargs)
 
 
 def post(*args, **kwargs):
     session = get_session(retries=3)
-    return session.post(*args, **kwargs)
+    return session.post(*args, timeout=timeout, **kwargs)
